/* 
 * tng_misc.h: miscellaneous Samba-TNG macros.
 * Copyright 2003 by Peter Samuelson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _TNG_MISC_H
#define _TNG_MISC_H

/*
 * STRIP_CONST lets you initialise a non-const pointer from a const
 * pointer, suppressing the usual compiler warning.
 *
 * You should ONLY use this if:
 *
 * (a) You are SURE that the contents of the pointer will NOT be
 *     modified, but
 * (b) There is no clean way to let the compiler know this.
 *
 * In particular, please do not use this macro as a quick fix to
 * suppress legitimate warnings.  It might be best to add a
 * comment at each instance, briefly explaining why it is safe.
 *
 * Usage: char *ch = STRIP_CONST(some_const_char_ptr);
 */
#if defined(__GNUC__) && __GNUC__ >= 2
#define STRIP_CONST(var) ({ \
	union { void **newptr; typeof(var) *oldptr; } _strip_c; \
	_strip_c.oldptr = &(var); \
	*_strip_c.newptr; \
})
#else
#define STRIP_CONST(var) (var)
#endif


/* memory-allocation-helpers (idea and names from glib) */
#define g_new(type, count) \
	((type *) malloc(sizeof(type) * (count)))
#define g_new0(type, count) \
	((type *) calloc((count), sizeof(type)))
#define g_renew(type, mem, count) \
	((type *) Realloc(mem, sizeof(type) * (count)))

/* Debugging helpers */
#define g_return_val_if_fail(expr, val) \
	do { \
		if (! (expr)) \
		{ \
			DEBUG(1, ("%s:%d(%s): %s failed\n", \
				  __FILE__, __LINE__, FUNCTION_MACRO, \
				  #expr)); \
			return val; \
		} \
	} while(0)


#endif /* _TNG_MISC_H */
