/* 
   Unix SMB/Netbios implementation.
   Functions useful in logging in people in the NT area
   Copyright (C) Elrond <elrond@samba-tng.org>  2002.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "includes.h"
#include "dlinklist.h"
#include "libsamba.h"
#include "tng_misc.h"
#include "rpc_client.h"
#include "others.h"

/****************************************************************************
  does _both_ nt->unix and unix->unix username remappings.
****************************************************************************/
const struct passwd *map_nt_and_unix_username(const char *domain,
					      const char *ntuser,
					      char *unix_user, char *nt_user)
{
	DOM_NAME_MAP gmep;
	fstring nt_username;
	if (nt_user == NULL)
	{
		nt_user = nt_username;
	}

	memset(nt_user, 0, sizeof(nt_user));
	if (domain != NULL)
	{
		slprintf(nt_user, sizeof(fstring), "%s\\%s", domain, ntuser);
	}
	else
	{
		fstrcpy(nt_user, ntuser);
	}

	/*
	 * Pass the user through the NT -> unix user mapping
	 * function.
	 */

	if (lookupsmbpwntnam(nt_user, &gmep))
	{
		fstrcpy(unix_user, gmep.unix_name);
	}
	else
	{
		return NULL;
	}

	/*
	 * Pass the user through the unix -> unix user mapping
	 * function.
	 */

	(void)map_username(unix_user);
	/*
	 * Do any UNIX username case mangling.
	 */
	return Get_Pwnam(unix_user, True);
}


/****************************************************************************/

struct _sid_list;

typedef struct _sid_list
{
	struct _sid_list *prev, *next;
	DOM_SID sid;
} SID_LIST_ENTRY;

static void create_base(SID_LIST_ENTRY **base, const NT_USER_TOKEN *token)
{
	size_t i;

	for (i = 0; i < token->num_sids; i++)
	{
		SID_LIST_ENTRY *l = g_new(SID_LIST_ENTRY, 1);
		if (! l)
			return;
		sid_copy(&l->sid, &token->user_sids[i]);
		DLIST_ADD((*base), l);
	}
}


static void do_the_fun(NT_USER_TOKEN *base_sids)
{
	size_t i;
	SID_LIST_ENTRY *list = NULL;
	SID_LIST_ENTRY *last_base, *start_entry;
	POLICY_HND sam_connect;

	create_base(&list, base_sids);

	last_base = list;

	if (! samr_connect("\\.", SEC_RIGHTS_MAXIMUM_ALLOWED, &sam_connect))
		return;

	start_entry = list;

	while(1)
	{
		size_t added = 0;
		
	}
}
