/* 
   Unix SMB/Netbios implementation.
   Misc and possibly useless stuff
   Copyright (C) Jeremy Allison 1992-1998
   Copyright (C) Elrond <elrond@samba-tng.org>  2001-2002.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/* MKPROTO: libsamba_proto.h		(do not remove this line) */

#include "includes.h"
#include <errno.h>
#ifdef HAVE_GLIB
#undef g_return_val_if_fail
#include <glib/gmessages.h>
#include <glib/gstring.h>
#endif
#include "libsamba.h"


/****************************************************************************
 protected memcpy that deals with NULL parameters.
 ****************************************************************************/
BOOL memcpy_zero(void *to, const void *from, size_t size)
{
	if (to == NULL)
	{
		return False;
	}

	if (size == 0)
	{
		DEBUG(5, ("memcpy_zero: Ignoring request to copy 0 bytes\n"));
		return True;
	}

	if (size > 16000000)
	{
		DEBUG(2, ("memcpy_zero: %u bytes, wee...\n", size));
	}

	if (from == NULL)
	{
		memset(to, 0, size);
		return False;
	}

	memcpy(to, from, size);
	return True;
}

/*********************************************************
 For rename across filesystems Patch from Warren Birnbaum 
 <warrenb@hpcvscdp.cv.hp.com>
**********************************************************/
#define BUFSIZE 4096
static int copy_reg(const char *source, const char *dest)
{
	SMB_STRUCT_STAT source_stats;
	int ifd = -1;
	int ofd = -1;
	char buf[BUFSIZE];
	int len;
	int saved_errno;
	struct utimbuf tv;

	sys_lstat(source, &source_stats);
	if (!S_ISREG(source_stats.st_mode))
		goto err;

	if ((ifd = sys_open(source, O_RDONLY, 0)) < 0)
		goto err;

	if (unlink(dest) && errno != ENOENT)
		goto err;

	if ((ofd = sys_open(dest, O_WRONLY | O_CREAT | O_EXCL, 0600)) < 0)
		goto err;

	while ((len = read(ifd, buf, BUFSIZE)) > 0)
	{
		if (write_data(ofd, buf, len) < 0)
			goto err;
	}
	if (len < 0)
		goto err;

	/* Try to preserve ownership.  For non-root it might fail, but that's ok.
	   But root probably wants to know, e.g. if NFS disallows it.  */
	if (fchown(ofd, source_stats.st_uid, source_stats.st_gid)
	    && (errno != EPERM))
		goto err;

	/* fchown turns off set[ug]id bits for non-root,
	   so do the chmod last.  */
	if (fchmod (ofd, source_stats.st_mode & 07777))
		goto err;

	if (close (ifd) < 0)
		goto err;
	ifd = -1;

	if (close (ofd) < 0)
		goto err;
	ofd = -1;

	/* Try to copy the old file's modtime and access time.  */
	tv.actime = source_stats.st_atime;
	tv.modtime = source_stats.st_mtime;
	if (utime(dest, &tv))
		goto err;

	unlink(source);
	return 0;

err:
	saved_errno = errno;
	if (ifd >= 0)
		close(ifd);
	if (ofd >= 0)
	{
		close(ofd);
		unlink(dest);
	}
	errno = saved_errno;
	return -1;
}

/*******************************************************************
  rename a unix file
********************************************************************/
int file_rename(char *from, char *to)
{
	int rcode = rename(from, to);

	if (errno == EXDEV)
	{
		/* Rename across filesystems needed. */
		rcode = copy_reg(from, to);
	}
	return rcode;
}

/*******************************************************************
prepare to dump a core file - carefully!
********************************************************************/
BOOL dump_core (void)
{
#if DUMP_CORE
	char *p;
	pstring dname;
	pstrcpy (dname, debugf);
	if ((p=strrchr (dname, '/'))) *p=0;
	pstrcat (dname, "/corefiles");
	mkdir (dname, 0700);
	sys_chown (dname, getuid(), getgid());
	chmod (dname, 0700);
	if (chdir (dname))
		return False;
	umask (~(0700));

#ifdef HAVE_GETRLIMIT
#ifdef RLIMIT_CORE
	{
		struct rlimit rlp;
		getrlimit (RLIMIT_CORE, &rlp);
		rlp.rlim_cur = MAX(4*1024*1024,rlp.rlim_cur);
		setrlimit (RLIMIT_CORE, &rlp);
		getrlimit (RLIMIT_CORE, &rlp);
		DEBUG(3,("Core limits now %d %d\n",
			 (int)rlp.rlim_cur, (int)rlp.rlim_max));
	}
#endif
#endif


	DEBUG(0,("Dumping core in %s\n",dname));
	abort ();
	return True;
#else
	return False;
#endif
}


/****************************************************************************
a wrapper for gethostbyname() that tries with all lower and all upper case 
if the initial name fails
****************************************************************************/
struct hostent *Get_Hostbyname(const char *name)
{
	char *name2 = strdup(name);
	struct hostent *ret;

	if (!name2)
	{
		DEBUG(0,
		      ("Memory allocation error in Get_Hostbyname! panic\n"));
		exit(0);
	}


	/* 
	 * This next test is redundent and causes some systems (with
	 * broken isalnum() calls) problems.
	 * JRA.
	 */

#if 0
	if (!isalnum(*name2))
	{
		free(name2);
		return (NULL);
	}
#endif /* 0 */

	ret = sys_gethostbyname(name2);
	if (ret != NULL)
	{
		free(name2);
		return (ret);
	}

	/* try with all lowercase */
	strlower(name2);
	ret = sys_gethostbyname(name2);
	if (ret != NULL)
	{
		free(name2);
		return (ret);
	}

	/* try with all uppercase */
	strupper(name2);
	ret = sys_gethostbyname(name2);
	if (ret != NULL)
	{
		free(name2);
		return (ret);
	}

	/* nothing works :-( */
	free(name2);
	return (NULL);
}


/*******************************************************************
 Reset the 'done' variables so after a client process is created
 from a fork call these calls will be re-done. This should be
 expanded if more variables need reseting.
 ******************************************************************/

void reset_globals_after_fork(void)
{
	/*
	 * Re-seed the random crypto generator, so all smbd's
	 * started from the same parent won't generate the same
	 * sequence.
	 */
	{
		unsigned char dummy;
		generate_random_buffer(&dummy, 1, True);
	}
}


/* needed by the next 2 functions, and by rpcclient/cmd_samr.c */
struct field_info acb_bitfield[] = {
	/* descriptions lifted from <smb.h> */
	{ ACB_PWNOTREQ,  "N", "User password not required" },
	{ ACB_DISABLED,  "D", "User account disabled" },
	{ ACB_HOMDIRREQ, "H", "Home directory required" },
	{ ACB_TEMPDUP,   "T", "Temporary duplicate account" },
	{ ACB_NORMAL,    "U", "Normal user account" },
	{ ACB_MNS,       "M", "MNS logon user account" },
	{ ACB_WSTRUST,   "W", "Workstation trust account" },
	{ ACB_SVRTRUST,  "S", "Server trust account" },
	{ ACB_AUTOLOCK,  "L", "Account locked" },
	{ ACB_PWNOEXP,   "X", "User password does not expire" },
	{ ACB_DOMTRUST,  "I", "Interdomain trust account" },
	{ ACB_PWLOCK,    "P", "Password is locked and cannot be changed remotely" },
	{ 0,             " ", NULL },
	{ 0, NULL, NULL }
};

/**********************************************************
 Encode the account control bits into a string.
 length = length of string to encode into (including terminating
 null). length *MUST BE MORE THAN 2* !
 **********************************************************/

char *pwdb_encode_acct_ctrl(uint32 acct_ctrl, size_t length)
{
	static fstring acct_str;
	size_t i;

	if (length > sizeof(acct_str))
	{
		DEBUG(0, ("BUG: pwdb_encode_acct_ctrl called with length=%d\n",
			  length));
		length = sizeof(acct_str);
	}

	acct_str[0] = '[';
	acct_str[1] = '\0';
	fstrcat (acct_str, bit_field_to_str (acct_ctrl, acb_bitfield, NULL));

	length -= 2;

	i = strlen(acct_str);
	if (i < length)
	{
		memset (acct_str+i, ' ', length-i);
	}

	i = length;

	acct_str[i++] = ']';
	acct_str[i++] = '\0';

	return acct_str;
}

/**********************************************************
 Decode the account control bits from a string.
 **********************************************************/

uint32 pwdb_decode_acct_ctrl(const char *p)
{
	/*
	 * Check if the account type bits have been encoded after the
	 * NT password (in the form [NDHTUWSLXI]).
	 */

	if (*p != '[')
		return 0;
	return str_to_bit_field (p+1, acb_bitfield, NULL);
}

/*************************************************************
 Routine to set 32 hex password characters from a 16 byte array.
**************************************************************/
void pwdb_sethexpwd(char *p, const uchar * pwd, uint16 acct_ctrl)
{
	if (pwd != NULL)
	{
		int i;
		for (i = 0; i < 16; i++)
		{
			slprintf(&p[i * 2], 33, "%02X", pwd[i]);
		}
	}
	else
	{
		if (IS_BITS_SET_ALL(acct_ctrl, ACB_PWNOTREQ))
		{
			unsafe_strcpy(p, "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX");
		}
		else
		{
			unsafe_strcpy(p, "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
		}
	}
}

/*************************************************************
 Routine to get the 32 hex characters and turn them
 into a 16 byte array.
**************************************************************/
BOOL pwdb_gethexpwd(const char *p, char *pwd, uint32 *acct_ctrl)
{
	if (strnequal(p, "NO PASSWORDXXXXXXXXXXXXXXXXXXXXX", 32))
	{
		if (acct_ctrl != NULL)
		{
			*acct_ctrl |= ACB_PWNOTREQ;
		}
		pwd[0] = 0;
		return True;
	}
	else if (strnequal(p, "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX", 32))
	{
		pwd[0] = 0;
		return True;
	}
	else
	{
		return strhex_to_str(pwd, 32, p) == 16;
	}
}


static struct field_info sid_name_info[] = {
	{SID_NAME_UNKNOWN, "UNKNOWN", NULL},	/* default */
	{SID_NAME_USER, "User", NULL},
	{SID_NAME_DOM_GRP, "Domain Group", NULL},
	{SID_NAME_DOMAIN, "Domain", NULL},
	{SID_NAME_ALIAS, "Local Group", NULL},
	{SID_NAME_WKN_GRP, "Well-known Group", NULL},
	{SID_NAME_DELETED, "Deleted", NULL},
	{SID_NAME_INVALID, "Invalid", NULL},
	{0, NULL, NULL}
};

/****************************************************************************
convert a SID_NAME_USE to a string 
****************************************************************************/
const char *get_sid_name_use_str(uint32 sid_name_use)
{
	return enum_field_to_str(sid_name_use, sid_name_info, True);
}


#ifdef HAVE_GLIB
static void
tng_glib_log_handler(const gchar   *log_domain,
		     GLogLevelFlags log_level,
		     const gchar   *message,
		     gpointer	    unused_data)
{
	const char *level_prefix = NULL;
	int level_num = 0;
	GString *gstring;

	switch (log_level & G_LOG_LEVEL_MASK)
	{
		case G_LOG_LEVEL_ERROR:
			level_prefix = "ERROR";
			break;
		case G_LOG_LEVEL_CRITICAL:
			level_prefix = "CRITICAL";
			break;
		case G_LOG_LEVEL_WARNING:
			level_prefix =  "WARNING";
			level_num = 1;
			break;
		case G_LOG_LEVEL_MESSAGE:
			level_num = 2;
			break;
		case G_LOG_LEVEL_INFO:
			level_num = 3;
			break;
		case G_LOG_LEVEL_DEBUG:
			level_num = 4;
			break;
		default:
			break;

	}

	gstring = g_string_new ("");
	if (log_domain)
	{
		g_string_append (gstring, log_domain);
		g_string_append_c (gstring, '-');
	}
	g_string_append (gstring, level_prefix);

	g_string_append (gstring, ": ");
	if (!message)
		g_string_append (gstring, "(NULL) message");
	else
	{
		/*
		 * The default glib handler does some UTF-8 converting here.
		 * I'm way too lazy to understand this now.
		 * And in reality, we don't have much intl stuff in glib handled
		 * messages...
		 */
		g_string_append (gstring, message);
	}
	g_string_append_c (gstring, '\n');

	DEBUG(level_num, ("%s", gstring->str));

	g_string_free(gstring, TRUE);
}
#endif


void init_glib_logging(void)
{
#ifdef HAVE_GLIB
	g_log_set_handler(NULL,
			  G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
			  tng_glib_log_handler, NULL);
	g_log_set_handler(G_LOG_DOMAIN,
			  G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
			  tng_glib_log_handler, NULL);
	g_log_set_handler("GLib",
			  G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION,
			  tng_glib_log_handler, NULL);
#endif
}
